@@ -1,12 +1,12 @@
 
 
 #' Stochastic gradient descent optimizer
-#' 
-#' Stochastic gradient descent optimizer with support for momentum, learning 
+#'
+#' Stochastic gradient descent optimizer with support for momentum, learning
 #' rate decay, and Nesterov momentum.
-#' 
-#' @param lr float >= 0. Learning rate.
-#' @param momentum float >= 0. Parameter that accelerates SGD in the relevant 
+#'
+#' @param learning_rate float >= 0. Learning rate.
+#' @param momentum float >= 0. Parameter that accelerates SGD in the relevant
 #'   direction and dampens oscillations.
 #' @param decay float >= 0. Learning rate decay over each update.
 #' @param nesterov boolean. Whether to apply Nesterov momentum.
@@ -14,19 +14,22 @@
 #'   value.
 #' @param clipvalue Gradients will be clipped when their absolute value exceeds
 #'   this value.
-#' 
+#' @param ... Unused, present only for backwards compatability
+#'
 #' @return Optimizer for use with \code{\link{compile.keras.engine.training.Model}}.
-#' 
-#' @family optimizers  
-#' 
+#'
+#' @family optimizers
+#'
 #' @export
-optimizer_sgd <- function(lr = 0.01, momentum = 0.0, decay = 0.0, nesterov = FALSE,
-                          clipnorm = NULL, clipvalue = NULL) {
-  
+optimizer_sgd <- function(learning_rate = 0.01, momentum = 0.0, decay = 0.0, nesterov = FALSE,
+                          clipnorm = NULL, clipvalue = NULL, ...) {
+
+  backcompat_fix_rename_lr_to_learning_rate(...)
+
   # compose args using list so that clipnorm and clipvalue are excluded
   # from the call when they aren't sepcified
   args <- list(
-    lr = lr,
+    learning_rate = learning_rate,
     momentum = momentum,
     decay = decay,
     nesterov = nesterov
@@ -37,25 +40,29 @@ optimizer_sgd <- function(lr = 0.01, momentum = 0.0, decay = 0.0, nesterov = FAL
 }
 
 #' RMSProp optimizer
-#' 
+#'
 #' @inheritParams optimizer_sgd
 #' @param rho float >= 0. Decay factor.
 #' @param epsilon float >= 0. Fuzz factor. If `NULL`, defaults to `k_epsilon()`.
-#' 
+#'
 #' @note It is recommended to leave the parameters of this optimizer at their
 #' default values (except the learning rate, which can be freely tuned).
-#' 
+#'
 #' This optimizer is usually a good choice for recurrent neural networks.
-#' 
-#' @family optimizers  
-#' 
+#'
+#' @family optimizers
+#'
 #' @export
-optimizer_rmsprop <- function(lr = 0.001, rho = 0.9, epsilon = NULL, decay = 0.0,
-                              clipnorm = NULL, clipvalue = NULL) {
+optimizer_rmsprop <- function(learning_rate = 0.001, rho = 0.9, epsilon = NULL, decay = 0.0,
+                              clipnorm = NULL, clipvalue = NULL, ...) {
+
+  backcompat_fix_rename_lr_to_learning_rate(...)
+
   # compose args using list so that clipnorm and clipvalue are excluded
   # from the call when they aren't sepcified
+
   args <- list(
-    lr = lr,
+    learning_rate = learning_rate,
     rho = rho,
     epsilon = resolve_epsilon(epsilon),
     decay = decay
@@ -80,12 +87,15 @@ optimizer_rmsprop <- function(lr = 0.001, rho = 0.9, epsilon = NULL, decay = 0.0
 #' @family optimizers
 #'
 #' @export
-optimizer_adagrad <- function(lr = 0.01, epsilon = NULL, decay = 0.0,
-                              clipnorm = NULL, clipvalue = NULL) {
+optimizer_adagrad <- function(learning_rate = 0.01, epsilon = NULL, decay = 0.0,
+                              clipnorm = NULL, clipvalue = NULL, ...) {
+
+  backcompat_fix_rename_lr_to_learning_rate(...)
+
   # compose args using list so that clipnorm and clipvalue are excluded
   # from the call when they aren't sepcified
   args <- list(
-    lr = lr,
+    learning_rate = learning_rate,
     epsilon = resolve_epsilon(epsilon),
     decay = decay
   )
@@ -107,12 +117,15 @@ optimizer_adagrad <- function(lr = 0.01, epsilon = NULL, decay = 0.0,
 #' @family optimizers
 #'
 #' @export
-optimizer_adadelta <- function(lr = 1.0, rho = 0.95, epsilon = NULL, decay = 0.0,
-                               clipnorm = NULL, clipvalue = NULL) {
+optimizer_adadelta <- function(learning_rate = 1.0, rho = 0.95, epsilon = NULL, decay = 0.0,
+                               clipnorm = NULL, clipvalue = NULL, ...) {
+
+  backcompat_fix_rename_lr_to_learning_rate(...)
+
   # compose args using list so that clipnorm and clipvalue are excluded
   # from the call when they aren't sepcified
   args <- list(
-    lr = lr,
+    learning_rate = learning_rate,
     rho = rho,
     epsilon = resolve_epsilon(epsilon),
     decay = decay
@@ -132,7 +145,7 @@ optimizer_adadelta <- function(lr = 1.0, rho = 0.95, epsilon = NULL, decay = 0.0
 #'   0 < beta < 1. Generally close to 1.
 #' @param beta_2 The exponential decay rate for the 2nd moment estimates. float,
 #'   0 < beta < 1. Generally close to 1.
-#' @param amsgrad Whether to apply the AMSGrad variant of this algorithm from 
+#' @param amsgrad Whether to apply the AMSGrad variant of this algorithm from
 #'   the paper "On the Convergence of Adam and Beyond".
 #'
 #' @note Default parameters follow those provided in the original paper.
@@ -144,12 +157,26 @@ optimizer_adadelta <- function(lr = 1.0, rho = 0.95, epsilon = NULL, decay = 0.0
 #' @family optimizers
 #'
 #' @export
-optimizer_adam <- function(lr = 0.001, beta_1 = 0.9, beta_2 = 0.999, epsilon = NULL, decay = 0.0,
-                           amsgrad = FALSE, clipnorm = NULL, clipvalue = NULL) {
+optimizer_adam <- function(learning_rate = 0.001, beta_1 = 0.9, beta_2 = 0.999, epsilon = NULL, decay = 0.0,
+                           amsgrad = FALSE, clipnorm = NULL, clipvalue = NULL, ...) {
+
+    backcompat_fix_rename_lr_to_learning_rate(...)
+  # TODO: name arg? decay position moved?
+
+  # TODO: lr -> learning_rate
+#   tf.keras.optimizers.Adam(
+#     learning_rate=0.001,
+#     beta_1=0.9,
+#     beta_2=0.999,
+#     epsilon=1e-07,
+#     amsgrad=False,
+#     name='Adam',
+#     **kwargs
+# )
   # compose args using list so that clipnorm and clipvalue are excluded
   # from the call when they aren't sepcified
   args <- list(
-    lr = lr,
+    learning_rate = learning_rate,
     beta_1 = beta_1,
     beta_2 = beta_2,
     epsilon = resolve_epsilon(epsilon),
@@ -157,29 +184,32 @@ optimizer_adam <- function(lr = 0.001, beta_1 = 0.9, beta_2 = 0.999, epsilon = N
   )
   args$clipnorm <- clipnorm
   args$clipvalue <- clipvalue
-  
+
   if (keras_version() >= "2.1.3")
     args$amsgrad <- amsgrad
-  
+
   do.call(keras$optimizers$Adam, args)
 }
 
 #' Adamax optimizer
-#' 
+#'
 #' Adamax optimizer from Section 7 of the [Adam paper](https://arxiv.org/abs/1412.6980v8).
 #' It is a variant of Adam based on the infinity norm.
-#' 
+#'
 #' @inheritParams optimizer_adam
-#' 
-#' @family optimizers  
-#' 
+#'
+#' @family optimizers
+#'
 #' @export
-optimizer_adamax <- function(lr = 0.002, beta_1 = 0.9, beta_2 = 0.999, epsilon = NULL, decay = 0.0,
-                             clipnorm = NULL, clipvalue = NULL) {
+optimizer_adamax <- function(learning_rate = 0.002, beta_1 = 0.9, beta_2 = 0.999, epsilon = NULL, decay = 0.0,
+                             clipnorm = NULL, clipvalue = NULL, ...) {
+
+  backcompat_fix_rename_lr_to_learning_rate(...)
+
   # compose args using list so that clipnorm and clipvalue are excluded
   # from the call when they aren't sepcified
   args <- list(
-    lr = lr,
+    learning_rate = learning_rate,
     beta_1 = beta_1,
     beta_2 = beta_2,
     epsilon = resolve_epsilon(epsilon),
@@ -187,7 +217,7 @@ optimizer_adamax <- function(lr = 0.002, beta_1 = 0.9, beta_2 = 0.999, epsilon =
   )
   args$clipnorm <- clipnorm
   args$clipvalue <- clipvalue
-  
+
   do.call(keras$optimizers$Adamax, args)
 }
 
@@ -209,12 +239,15 @@ optimizer_adamax <- function(lr = 0.002, beta_1 = 0.9, beta_2 = 0.999, epsilon =
 #' @family optimizers
 #'
 #' @export
-optimizer_nadam <- function(lr = 0.002, beta_1 = 0.9, beta_2 = 0.999, epsilon = NULL, 
-                            schedule_decay = 0.004, clipnorm = NULL, clipvalue = NULL) {
+optimizer_nadam <- function(learning_rate = 0.002, beta_1 = 0.9, beta_2 = 0.999, epsilon = NULL,
+                            schedule_decay = 0.004, clipnorm = NULL, clipvalue = NULL, ...) {
+
+  backcompat_fix_rename_lr_to_learning_rate(...)
+
   # compose args using list so that clipnorm and clipvalue are excluded
   # from the call when they aren't sepcified
   args <- list(
-    lr = lr,
+    learning_rate = learning_rate,
     beta_1 = beta_1,
     beta_2 = beta_2,
     epsilon = resolve_epsilon(epsilon),
@@ -222,13 +255,23 @@ optimizer_nadam <- function(lr = 0.002, beta_1 = 0.9, beta_2 = 0.999, epsilon =
   )
   args$clipnorm <- clipnorm
   args$clipvalue <- clipvalue
-  
+
   do.call(keras$optimizers$Nadam, args)
 }
 
 resolve_epsilon <- function(epsilon) {
   if (is.null(epsilon) && keras_version() < "2.1.3")
     k_epsilon()
-  else 
+  else
     epsilon
 }
+
+backcompat_fix_rename_lr_to_learning_rate <- function(..., lr) {
+  if (!missing(lr)) {
+      warning("the `lr` argument has been renamed to `learning_rate`.")
+      if (!eval.parent(quote(missing(learning_rate))))
+        stop("You can't supply both `lr` and `learning_rate`")
+      assign("learning_rate", lr, parent.frame())
+  }
+  ellipsis::check_dots_empty()
+}
